@@ -1,10 +1,13 @@
 """Tools for managing kernel specs"""
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
+
 import json
 import os
 import re
 import shutil
+import typing as t
 import warnings
 
 from jupyter_core.paths import SYSTEM_JUPYTER_PATH, jupyter_data_dir, jupyter_path
@@ -32,7 +35,7 @@ class KernelSpec(HasTraits):
     metadata = Dict()
 
     @classmethod
-    def from_resource_dir(cls, resource_dir):
+    def from_resource_dir(cls: type[KernelSpec], resource_dir: str) -> KernelSpec:
         """Create a KernelSpec object by reading kernel.json
 
         Pass the path to the *directory* containing kernel.json.
@@ -42,7 +45,7 @@ class KernelSpec(HasTraits):
             kernel_dict = json.load(f)
         return cls(resource_dir=resource_dir, **kernel_dict)
 
-    def to_dict(self):
+    def to_dict(self) -> dict[str, t.Any]:
         """Convert the kernel spec to a dict."""
         d = {
             "argv": self.argv,
@@ -55,7 +58,7 @@ class KernelSpec(HasTraits):
 
         return d
 
-    def to_json(self):
+    def to_json(self) -> str:
         """Serialise this kernelspec to a JSON object.
 
         Returns a string.
@@ -66,7 +69,7 @@ class KernelSpec(HasTraits):
 _kernel_name_pat = re.compile(r"^[a-z0-9._\-]+$", re.IGNORECASE)
 
 
-def _is_valid_kernel_name(name):
+def _is_valid_kernel_name(name: str) -> t.Any:
     """Check that a kernel name is valid."""
     # quote is not unicode-safe on Python 2
     return _kernel_name_pat.match(name)
@@ -78,12 +81,12 @@ _kernel_name_description = (
 )
 
 
-def _is_kernel_dir(path):
+def _is_kernel_dir(path: str) -> bool:
     """Is ``path`` a kernel directory?"""
     return os.path.isdir(path) and os.path.isfile(pjoin(path, "kernel.json"))
 
 
-def _list_kernels_in(dir):
+def _list_kernels_in(dir: str | None) -> dict[str, str]:
     """Return a mapping of kernel names to resource directories from dir.
 
     If dir is None or does not exist, returns an empty dict.
@@ -108,11 +111,11 @@ def _list_kernels_in(dir):
 class NoSuchKernel(KeyError):  # noqa
     """An error raised when there is no kernel of a give name."""
 
-    def __init__(self, name):
+    def __init__(self, name: str) -> None:
         """Initialize the error."""
         self.name = name
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"No such kernel named {self.name}"
 
 
@@ -137,12 +140,12 @@ class KernelSpecManager(LoggingConfigurable):
 
     data_dir = Unicode()
 
-    def _data_dir_default(self):
+    def _data_dir_default(self) -> str:
         return jupyter_data_dir()
 
     user_kernel_dir = Unicode()
 
-    def _user_kernel_dir_default(self):
+    def _user_kernel_dir_default(self) -> str:
         return pjoin(self.data_dir, "kernels")
 
     whitelist = Set(
@@ -168,7 +171,7 @@ class KernelSpecManager(LoggingConfigurable):
     # Method copied from
     # https://github.com/jupyterhub/jupyterhub/blob/d1a85e53dccfc7b1dd81b0c1985d158cc6b61820/jupyterhub/auth.py#L143-L161
     @observe(*list(_deprecated_aliases))
-    def _deprecated_trait(self, change):
+    def _deprecated_trait(self, change: t.Any) -> None:
         """observer for deprecated traits"""
         old_attr = change.name
         new_attr, version = self._deprecated_aliases[old_attr]
@@ -183,7 +186,7 @@ class KernelSpecManager(LoggingConfigurable):
             )
             setattr(self, new_attr, change.new)
 
-    def _kernel_dirs_default(self):
+    def _kernel_dirs_default(self) -> list[str]:
         dirs = jupyter_path("kernels")
         # At some point, we should stop adding .ipython/kernels to the path,
         # but the cost to keeping it is very small.
@@ -196,7 +199,7 @@ class KernelSpecManager(LoggingConfigurable):
             pass
         return dirs
 
-    def find_kernel_specs(self):
+    def find_kernel_specs(self) -> dict[str, str]:
         """Returns a dict mapping kernel names to resource directories."""
         d = {}
         for kernel_dir in self.kernel_dirs:
@@ -225,7 +228,7 @@ class KernelSpecManager(LoggingConfigurable):
         return d
         # TODO: Caching?
 
-    def _get_kernel_spec_by_name(self, kernel_name, resource_dir):
+    def _get_kernel_spec_by_name(self, kernel_name: str, resource_dir: str) -> KernelSpec:
         """Returns a :class:`KernelSpec` instance for a given kernel_name
         and resource_dir.
         """
@@ -238,7 +241,8 @@ class KernelSpecManager(LoggingConfigurable):
                 pass
             else:
                 if resource_dir == RESOURCES:
-                    kspec = self.kernel_spec_class(resource_dir=resource_dir, **get_kernel_dict())
+                    kdict = get_kernel_dict()  # type:ignore[no-untyped-call]
+                    kspec = self.kernel_spec_class(resource_dir=resource_dir, **kdict)
         if not kspec:
             kspec = self.kernel_spec_class.from_resource_dir(resource_dir)
 
@@ -247,7 +251,7 @@ class KernelSpecManager(LoggingConfigurable):
 
         return kspec
 
-    def _find_spec_directory(self, kernel_name):
+    def _find_spec_directory(self, kernel_name: str) -> str | None:
         """Find the resource directory of a named kernel spec"""
         for kernel_dir in [kd for kd in self.kernel_dirs if os.path.isdir(kd)]:
             files = os.listdir(kernel_dir)
@@ -263,8 +267,9 @@ class KernelSpecManager(LoggingConfigurable):
                 pass
             else:
                 return RESOURCES
+        return None
 
-    def get_kernel_spec(self, kernel_name):
+    def get_kernel_spec(self, kernel_name: str) -> KernelSpec:
         """Returns a :class:`KernelSpec` instance for the given kernel_name.
 
         Raises :exc:`NoSuchKernel` if the given kernel name is not found.
@@ -281,7 +286,7 @@ class KernelSpecManager(LoggingConfigurable):
 
         return self._get_kernel_spec_by_name(kernel_name, resource_dir)
 
-    def get_all_specs(self):
+    def get_all_specs(self) -> dict[str, t.Any]:
         """Returns a dict mapping kernel names to kernelspecs.
 
         Returns a dict of the form::
@@ -313,7 +318,7 @@ class KernelSpecManager(LoggingConfigurable):
                 self.log.warning("Error loading kernelspec %r", kname, exc_info=True)
         return res
 
-    def remove_kernel_spec(self, name):
+    def remove_kernel_spec(self, name: str) -> str:
         """Remove a kernel spec directory by name.
 
         Returns the path that was deleted.
@@ -332,7 +337,9 @@ class KernelSpecManager(LoggingConfigurable):
             shutil.rmtree(spec_dir)
         return spec_dir
 
-    def _get_destination_dir(self, kernel_name, user=False, prefix=None):
+    def _get_destination_dir(
+        self, kernel_name: str, user: bool = False, prefix: str | None = None
+    ) -> str:
         if user:
             return os.path.join(self.user_kernel_dir, kernel_name)
         elif prefix:
@@ -341,8 +348,13 @@ class KernelSpecManager(LoggingConfigurable):
             return os.path.join(SYSTEM_JUPYTER_PATH[0], "kernels", kernel_name)
 
     def install_kernel_spec(
-        self, source_dir, kernel_name=None, user=False, replace=None, prefix=None
-    ):
+        self,
+        source_dir: str,
+        kernel_name: str | None = None,
+        user: bool = False,
+        replace: bool | None = None,
+        prefix: str | None = None,
+    ) -> str:
         """Install a kernel spec by copying its directory.
 
         If ``kernel_name`` is not given, the basename of ``source_dir`` will
@@ -395,7 +407,7 @@ class KernelSpecManager(LoggingConfigurable):
         self.log.info("Installed kernelspec %s in %s", kernel_name, destination)
         return destination
 
-    def install_native_kernel_spec(self, user=False):
+    def install_native_kernel_spec(self, user: bool = False) -> None:
         """DEPRECATED: Use ipykernel.kernelspec.install"""
         warnings.warn(
             "install_native_kernel_spec is deprecated. Use ipykernel.kernelspec import install.",
@@ -403,15 +415,15 @@ class KernelSpecManager(LoggingConfigurable):
         )
         from ipykernel.kernelspec import install
 
-        install(self, user=user)
+        install(self, user=user)  # type:ignore[no-untyped-call]
 
 
-def find_kernel_specs():
+def find_kernel_specs() -> dict[str, str]:
     """Returns a dict mapping kernel names to resource directories."""
     return KernelSpecManager().find_kernel_specs()
 
 
-def get_kernel_spec(kernel_name):
+def get_kernel_spec(kernel_name: str) -> KernelSpec:
     """Returns a :class:`KernelSpec` instance for the given kernel_name.
 
     Raises KeyError if the given kernel name is not found.
@@ -419,7 +431,13 @@ def get_kernel_spec(kernel_name):
     return KernelSpecManager().get_kernel_spec(kernel_name)
 
 
-def install_kernel_spec(source_dir, kernel_name=None, user=False, replace=False, prefix=None):
+def install_kernel_spec(
+    source_dir: str,
+    kernel_name: str | None = None,
+    user: bool = False,
+    replace: bool | None = False,
+    prefix: str | None = None,
+) -> str:
     """Install a kernel spec in a given directory."""
     return KernelSpecManager().install_kernel_spec(source_dir, kernel_name, user, replace, prefix)
 
@@ -427,9 +445,9 @@ def install_kernel_spec(source_dir, kernel_name=None, user=False, replace=False,
 install_kernel_spec.__doc__ = KernelSpecManager.install_kernel_spec.__doc__
 
 
-def install_native_kernel_spec(user=False):
+def install_native_kernel_spec(user: bool = False) -> None:
     """Install the native kernel spec."""
-    return KernelSpecManager().install_native_kernel_spec(user=user)
+    KernelSpecManager().install_native_kernel_spec(user=user)
 
 
 install_native_kernel_spec.__doc__ = KernelSpecManager.install_native_kernel_spec.__doc__
