@@ -76,7 +76,7 @@ def in_pending_state(method: F) -> F:
 
     @t.no_type_check
     @functools.wraps(method)
-    async def wrapper(self, *args, **kwargs):
+    async def wrapper(self: t.Any, *args: t.Any, **kwargs: t.Any) -> t.Any:
         """Create a future for the decorated method."""
         if self._attempted_start or not self._ready:
             self._ready = _get_future()
@@ -104,7 +104,7 @@ class KernelManager(ConnectionFileMixin):
 
     _ready: t.Optional[t.Union[Future, CFuture]]
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: t.Any, **kwargs: t.Any) -> None:
         """Initialize a kernel manager."""
         self._owns_kernel = kwargs.pop("owns_kernel", True)
         super().__init__(**kwargs)
@@ -268,6 +268,21 @@ class KernelManager(ConnectionFileMixin):
     # Kernel management
     # --------------------------------------------------------------------------
 
+    def update_env(self, *, env: t.Dict[str, str]) -> None:
+        """
+        Allow to update the environment of a kernel manager.
+
+        This will take effect only after kernel restart when the new env is
+        passed to the new kernel.
+
+        This is useful as some of the information of the current kernel reflect
+        the state of the session that started it, and those session information
+        (like the attach file path, or name), are mutable.
+
+        .. version-added: 8.5
+        """
+        self._launch_args['env'].update(env)
+
     def format_kernel_cmd(self, extra_arguments: t.Optional[t.List[str]] = None) -> t.List[str]:
         """Replace templated args (e.g. {connection_file})"""
         extra_arguments = extra_arguments or []
@@ -304,7 +319,7 @@ class KernelManager(ConnectionFileMixin):
 
         pat = re.compile(r"\{([A-Za-z0-9_]+)\}")
 
-        def from_ns(match):
+        def from_ns(match: t.Any) -> t.Any:
             """Get the key out of ns if it's there, otherwise no change."""
             return ns.get(match.group(1), match.group())
 
